'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : async.vb
'--------------------------------------------------------------------------
' Komodo asynchronous test routine
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================
Imports TotalPhase

Module async
    '==========================================================================
    ' FUNCTIONS
    '==========================================================================
    Public Sub _async(ByRef km As Integer, _
                      ByRef can_id As Integer, _
                      ByRef can_ch As km_can_ch_t, _
                      ByRef iter As Integer)
        Dim i As Integer
        Dim ret As Integer
        Dim timeout As Integer = 500
        Dim data_len As Integer = 8
        Dim data() As Byte = {1, 2, 3, 4, 5, 6, 7, 8}
        Dim arb_count As UInteger = 0

        Dim pkt As New KomodoApi.km_can_packet_t
        pkt.remote_req = 0
        pkt.extend_addr = 0
        pkt.dlc = data_len
        pkt.id = &HAB

        ' Enable Komodo
        ret = KomodoApi.km_enable(km)
        If ret <> km_status_t.KM_OK Then
            Console.WriteLine("Unable to enable Komodo")
            Exit Sub
        End If

        ' Send first packet
        KomodoApi.km_can_async_submit(km, can_ch, 0, pkt, data_len, data)
        Console.WriteLine("Submitted packet 1")

        For i = 0 To iter - 2
            ' Send another packet while the first is still in progress
            KomodoApi.km_can_async_submit(km, can_ch, 0, pkt, data_len, data)
            Console.WriteLine("Submitted packet " & i + 2)

            ' The application can now perform other functions 
            ' while the Komodo is sending the previous packet
            KomodoApi.km_sleep_ms(100)

            ' Collect response to previously submitted packet
            ret = KomodoApi.km_can_async_collect(km, timeout, arb_count)

            If ret = km_status_t.KM_OK Then
                Console.WriteLine("Collected packet " & i + 1)
            Else
                Console.WriteLine("Error " & ret)
            End If
        Next

        ' Collect the last response
        ret = KomodoApi.km_can_async_collect(km, timeout, arb_count)
        If ret = km_status_t.KM_OK Then
            Console.WriteLine("Collected packet " & i + 1)
        Else
            Console.WriteLine("Error " & ret)
        End If
    End Sub

    '==========================================================================
    ' MAIN PROGRAM
    '==========================================================================
    Public Sub async_run()
        Dim bitrate As Integer = 125000 ' Hz
        Dim port As Integer = 0
        Dim iter As Integer = 10
        Dim can_id As Integer = 66
        Dim ret As Integer
        Dim km As Integer

        ' Open the interface
        km = KomodoApi.km_open(port)
        If km <= 0 Then
            Console.WriteLine("Unable to open Komodo on port {0:d}", port)
            Console.WriteLine("Error code = {0:d}", km)
            Exit Sub
        End If

        ' Aquire freatures
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG Or _
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL)
        Console.WriteLine("Aquired features: 0x{0:x}", ret)

        ' Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate)
        Console.Write("Bitrate set to " & ret / 1000 & "kHz, ")

        ' Set target power
        KomodoApi.km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, _
                                      km_power_t.KM_TARGET_POWER_ON)
        Console.WriteLine("Enabled target power for CAN A")

        _async(km, can_id, km_can_ch_t.KM_CAN_CH_A, iter)
        Console.WriteLine()

        ' Close and exit
        KomodoApi.km_close(km)
    End Sub
End Module
