'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : gpio.vb
'--------------------------------------------------------------------------
' Komodo GPIO test routine
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================
Imports TotalPhase

Module gpio
    '==========================================================================
    ' FUNCTIONS
    '==========================================================================
    Public Sub show_pin_state(ByRef state As Integer, ByRef pin As Integer)
        Dim val As String
        Dim mask As Integer = (1 << (pin - 1))

        If state And mask Then
            val = "high"
        Else
            val = "low"
        End If
        Console.WriteLine("Pin " & pin & " is " & val)
    End Sub

    Public Sub monitor_changes(ByRef km As Integer)
        Dim i As Integer
        Dim oldval As Integer
        Dim endtime As DateTime
        Dim num_gpios As Integer = 8
        Dim wait_sec As Integer = 20

        Console.WriteLine("Configuring pins as inputs with pullups")
        For i = 0 To num_gpios - 1
            KomodoApi.km_gpio_config_in(km, i, km_pin_bias_t.KM_PIN_BIAS_PULLUP, _
                                        km_pin_trigger_t.KM_PIN_TRIGGER_NONE)
        Next

        ' Enable Komodo
        KomodoApi.km_enable(km)
        Console.WriteLine("Monitoring GPIO pin changes for " & wait_sec & " seconds:")

        endtime = Now.AddSeconds(20)
        oldval = KomodoApi.km_gpio_get(km)

        While Now < endtime
            Dim change As Integer
            Dim newval As Integer

            ' Check for pin changes
            newval = KomodoApi.km_gpio_get(km)
            If newval <> oldval Then
                change = oldval Xor newval

                ' Display the pins that changed
                For i = 0 To num_gpios - 1
                    If change And (1 << i) Then
                        show_pin_state(newval, i + 1)
                    End If
                Next

               ' Save the current state
               oldval = newval
            End If
        End While

    End Sub

    Public Sub _gpio(ByRef km As Integer)
        Dim ret As Integer

        Console.WriteLine("Configuring Pin 1 as an output")
        KomodoApi.km_gpio_config_out(km, KomodoApi.KM_GPIO_PIN_1_CONFIG, _
                                     km_pin_drive_t.KM_PIN_DRIVE_NORMAL, _
                                     km_pin_source_t.KM_PIN_SRC_SOFTWARE_CTL)

        Console.WriteLine("Configure Pin 2 as an input with pullup")
        KomodoApi.km_gpio_config_in(km, KomodoApi.KM_GPIO_PIN_2_CONFIG, _
                                    km_pin_bias_t.KM_PIN_BIAS_PULLUP, _
                                    km_pin_trigger_t.KM_PIN_TRIGGER_NONE)

        ' Enable Komodo
        KomodoApi.km_enable(km)

        Console.WriteLine("Setting Pin 1 to logic low")
        KomodoApi.km_gpio_set(km, 0, KomodoApi.KM_GPIO_PIN_1_MASK)

        ' Show the state of each pin
        ' Pin 1 should be low because it was set low
        ' Pin 2 should be high because it's an input with a pullup resistor
        ret = KomodoApi.km_gpio_get(km)
        show_pin_state(ret, 1)
        show_pin_state(ret, 2)

        ' Disable the Komodo to change the GPIO configuration
        KomodoApi.km_disable(km)

        Console.WriteLine("Configuring Pin 2 set as an input with pulldown")
        KomodoApi.km_gpio_config_in(km, KomodoApi.KM_GPIO_PIN_2_CONFIG, _
                                    km_pin_bias_t.KM_PIN_BIAS_PULLDOWN, _
                                    km_pin_trigger_t.KM_PIN_TRIGGER_NONE)

        ' Enable and show value of Pin 2
        KomodoApi.km_enable(km)
        ret = KomodoApi.km_gpio_get(km)
        show_pin_state(ret, 2)
        KomodoApi.km_disable(km)

        ' Set GPIOs as input and monitor for changes
        monitor_changes(km)
    End Sub

    '==========================================================================
    ' MAIN PROGRAM
    '==========================================================================
    Public Sub gpio_run()
        Dim km As Integer
        Dim ret As Integer
        Dim port As Integer = 0

        ' Open the interface
        km = KomodoApi.km_open(port)
        If km <= 0 Then
            Console.WriteLine("Unable to open Komodo on port {0:d}", port)
            Console.WriteLine("Error code = {0:d}", km)
            Exit Sub
        End If

        ' Aquire freatures
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_GPIO_LISTEN Or _
                                       KomodoApi.KM_FEATURE_GPIO_CONTROL Or _
                                       KomodoApi.KM_FEATURE_GPIO_CONFIG)
        Console.WriteLine("Aquired features: 0x{0:x}", ret)


        _gpio(km)
        Console.WriteLine()

        ' Close and exit
        KomodoApi.km_close(km)
    End Sub
End Module
