'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : loopback.vb
'--------------------------------------------------------------------------
' CAN loopback test routine
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================
Imports TotalPhase

Module loopback
    '======================================================================
    ' UTILITY FUNCTIONS
    '======================================================================
    Sub print_num_array (ByRef array() As Byte, ByVal len As Integer)
        Dim i as Integer

        Console.Write("[ ")
        For i = 0 To len - 1
            Console.Write("{0:x} ", array(i))
        Next
        Console.Write("]")
    End Sub

    Sub _loopback (ByRef kma As Integer, ByRef kmb As Integer)
        Dim arb_count         as UInteger = 0
        Dim ret               as Integer
        Dim i                 as Integer
        Dim data_len          as Integer = 8
        Dim data_out()        as Byte    = { 1, 2, 3, 4, 5, 6, 7, 8 }
        Dim data_in(data_len) as Byte

        Dim info as New KomodoApi.km_can_info_t

        Dim pkt as New KomodoApi.km_can_packet_t
        pkt.remote_req  = 0
        pkt.extend_addr = 0
        pkt.dlc         = data_len
        pkt.id          = &HAB

        ' Enable CAN A and B
        ret = KomodoApi.km_enable(kma)
        If ret <> km_status_t.KM_OK
            Console.WriteLine("Unable to enable Komodo A")
            Exit Sub
        End If

        ret = KomodoApi.km_enable(kmb)
        If ret <> km_status_t.KM_OK
            Console.WriteLine("Unable to enable Komodo B")
            Exit Sub
        End If

        ' Send data on CAN A
        KomodoApi.km_can_write(kma, km_can_ch_t.KM_CAN_CH_A, 0, _
                               pkt, data_len, data_out, arb_count)

        Console.WriteLine()
        Console.Write("Sent data:     ")
        print_num_array(data_out, data_len)

        ' Read data on CAN B
        Do While ret = 0
            ret = KomodoApi.km_can_read(kmb, info, pkt, data_len, data_in)
        Loop

        Console.WriteLine()
        Console.Write("Received data: ")
        print_num_array(data_in, ret)

        Console.WriteLine()
        Console.Write("Verifying data... ")

        ' Verify received data
        For i = 0 to data_len - 1
            If data_out(i) <> data_in(i) Then
               Console.WriteLine("FAIL")
               Exit Sub
            End If
        Next

        Console.WriteLine("PASS")
    End Sub


    '======================================================================
    ' MAIN PROGRAM
    '======================================================================
    Public Sub loopback_run()
        Dim bitrate      as Integer = 125000 ' Hz
        Dim timeout      as Integer = 1000   ' ms
        Dim reta,  retb  as Integer
        Dim kma,   kmb   as Integer
        Dim porta, portb as Integer

        porta = 0
        portb = 1

        kma = KomodoApi.km_open(porta)
        If kma <= 0
            Console.WriteLine("Unable to open Komodo on port {0:d}", porta)
            Console.WriteLine("Error code = {0:d}", kma)
            Exit Sub
        End If

        kmb = KomodoApi.km_open(portb)
        If kmb <= 0
            Console.WriteLine("Unable to open Komodo on port {0:d}", portb)
            Console.WriteLine("Error code = {0:d}", kmb)
            Exit Sub
        End If

        'Setup CAN channel A for writing and B for listening
        reta = KomodoApi.km_acquire(kma, KomodoApi.KM_FEATURE_CAN_A_CONFIG  Or _
                                         KomodoApi.KM_FEATURE_CAN_A_CONTROL)
        retb = KomodoApi.km_acquire(kmb, KomodoApi.KM_FEATURE_CAN_B_CONFIG  Or _
                                         KomodoApi.KM_FEATURE_CAN_B_CONTROL Or _
                                         KomodoApi.KM_FEATURE_CAN_B_LISTEN)
        Console.WriteLine("Features for CAN A: 0x{0:x}, CAN B: 0x{1:x}", _
                          reta, retb)

        ' Set bitrate
        reta = KomodoApi.km_can_bitrate(kma, km_can_ch_t.KM_CAN_CH_A, bitrate)
        retb = KomodoApi.km_can_bitrate(kmb, km_can_ch_t.KM_CAN_CH_B, bitrate)
        Console.Write("Bitrate for CAN A set to " & reta/1000 & "kHz, ")
        Console.WriteLine("CAN B set to " & retb/1000 & "kHz")

        ' Set timeout
        KomodoApi.km_timeout(kmb, timeout)
        Console.WriteLine("Timeout for CAN B set to {0:d} ms", timeout)

        ' Set target power
        KomodoApi.km_can_target_power(kma, km_can_ch_t.KM_CAN_CH_A, _
                                      km_power_t.KM_TARGET_POWER_ON)
        KomodoApi.km_can_target_power(kmb, km_can_ch_t.KM_CAN_CH_B, _
                                      km_power_t.KM_TARGET_POWER_OFF)
        Console.WriteLine("Enabled target power for CAN A " + _
                          "and disabled for CAN B")

        _loopback(kma, kmb)
        Console.WriteLine()

        ' Close and exit
        KomodoApi.km_close(kma)
        KomodoApi.km_close(kmb)

    End Sub
End Module
