'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : monitor.vb
'--------------------------------------------------------------------------
' CAN monitor test routine
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES
' LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================
Imports TotalPhase

Module monitor
    '======================================================================
    ' UTILITY FUNCTIONS
    '======================================================================
    Function timestamp_to_ns(ByVal stamp As ULong, _
                              ByVal samplerate_khz As Integer) As ULong
        Return (stamp * 1000 / (samplerate_khz / 1000))
    End Function

    Sub print_num_array(ByRef array() As Byte, ByVal len As Integer)
        Dim i As Integer

        For i = 0 To len - 1
            Console.Write("{0:x} ", array(i))
        Next
        Console.Out.Flush()
    End Sub

    Sub print_status(ByVal status As UInteger)
        Console.Write(" ")

        If (status = km_status_t.KM_OK) Then
            Console.Write("OK ")
        End If
        If ((status And KomodoApi.KM_READ_TIMEOUT) <> 0) Then
            Console.Write("TIMEOUT ")
        End If
        If ((status And KomodoApi.KM_READ_ERR_OVERFLOW) <> 0) Then
            Console.Write("OVERFLOW ")
        End If
        If ((status And KomodoApi.KM_READ_END_OF_CAPTURE) <> 0) Then
            Console.Write("END OF CAPTURE ")
        End If
        If ((status And KomodoApi.KM_READ_CAN_ARB_LOST) <> 0) Then
            Console.Write("ARBITRATION LOST ")
        End If
        If ((status And KomodoApi.KM_READ_CAN_ERR) <> 0) Then
            Console.Write("ERROR {0:x} ", _
                          status And KomodoApi.KM_READ_CAN_ERR_FULL_MASK)
        End If
    End Sub

    Sub print_events(ByVal events As UInteger, ByVal bitrate As UInteger)
        If (events = 0) Then
            Exit Sub
        End If
        If ((events And KomodoApi.KM_EVENT_DIGITAL_INPUT) <> 0) Then
            Console.Write("GPIO CHANGE 0x{0:x}; ", _
                   events And KomodoApi.KM_EVENT_DIGITAL_INPUT_MASK)
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_LISTEN_ONLY) <> 0) Then
            Console.Write("BUS STATE LISTEN ONLY; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_CONTROL) <> 0) Then
            Console.Write("BUS STATE CONTROL; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_WARNING) <> 0) Then
            Console.Write("BUS STATE WARNING; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_ACTIVE) <> 0) Then
            Console.Write("BUS STATE ACTIVE; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_PASSIVE) <> 0) Then
            Console.Write("BUS STATE PASSIVE; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_STATE_OFF) <> 0) Then
            Console.Write("BUS STATE OFF; ")
        End If
        If ((events And KomodoApi.KM_EVENT_CAN_BUS_BITRATE) <> 0) Then
            Console.Write("BITRATE " & bitrate / 1000 & "kHz; ")
        End If
    End Sub

    Sub candump(ByVal km As Integer, ByVal max_events As Integer)
        Dim ret As Integer
        Dim count As Integer
        Dim samplerate_khz As Integer

        Dim data_len As Integer = 8
        Dim data(data_len) As Byte

        Dim info As New KomodoApi.km_can_info_t
        Dim pkt As New KomodoApi.km_can_packet_t

        ' Get samplerate
        samplerate_khz = KomodoApi.km_get_samplerate(km) / 1000

        ' Enable Komodo
        ret = KomodoApi.km_enable(km)
        If (ret <> km_status_t.KM_OK) Then
            Console.WriteLine("Unable to enable Komodo")
            Exit Sub
        End If

        ' Print description of csv output
        Console.WriteLine()
        Console.WriteLine("index,time(ns),(status & events),<ID:rtr/data> hex data")
        Console.WriteLine()

        ' Start monitoring. Default to infinite events.
        ' To stop program after N events, change "count" below

        count = 0

        While ((max_events = 0) Or (count < max_events))
            count += 1
            ret = KomodoApi.km_can_read(km, info, pkt, data_len, data)

            Console.Write("{0:d},{1:d},(", count, _
                           timestamp_to_ns(info.timestamp, samplerate_khz))

            If ret < 0 Then
                Console.Write("error={0:d}", ret)
            End If

            print_status(info.status)
            print_events(info.events, info.bitrate_hz)
            Console.Write(")")

            ' Continue printing if we didn't see timeout
            ' error or dataless (events)
            If ((info.status = km_status_t.KM_OK) And (info.events = 0)) Then
                Console.Write(",<{0:x}:{1:s} ", pkt.id, _
                              IIf(pkt.remote_req, "rtr>", "data>"))

                ' If packet contained data, print it
                If pkt.remote_req = 0 Then
                    print_num_array(data, pkt.dlc)
                End If
            End If

            Console.WriteLine("")
            Console.Out.Flush()
        End While
    End Sub


    '======================================================================
    ' MAIN PROGRAM
    '======================================================================
    Public Sub monitor_run()
        Dim km As Integer
        Dim i As Byte
        Dim ret As Integer
        Dim power As km_power_t
        Dim max_events As Integer
        Dim port As Integer = 0          ' Use port 0
        Dim timeout As UShort = 1000     ' ms
        Dim bitrate As UInteger = 125000 ' Hz
        Dim NUM_GPIOS As Integer = 8

        power = km_power_t.KM_TARGET_POWER_ON
        max_events = 0

        ' Open the interface
        km = KomodoApi.km_open(port)
        If km <= 0 Then
            Console.WriteLine("Unable to open Komodo on port {0:d}", port)
            Console.WriteLine("Error code = {0:d}", km)
            Exit Sub
        End If

        ' Acquire features.  Acquiring KM_FEATURE_CAN_A_CONTROL causes the
        ' Komodo interface to ACK all packets transmitted on the bus.  Remove
        ' this feature to prevent the device from transmitting anything on the
        ' bus.
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG Or _
                                       KomodoApi.KM_FEATURE_CAN_A_LISTEN Or _
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL Or _
                                       KomodoApi.KM_FEATURE_GPIO_CONFIG Or _
                                       KomodoApi.KM_FEATURE_GPIO_LISTEN)
        Console.WriteLine("Acquired features 0x{0:x}", ret)

        ' Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate)
        Console.WriteLine("Bitrate set to" & ret / 1000 & "kHz")

        ' Set timeout
        KomodoApi.km_timeout(km, timeout)
        Console.WriteLine("Timeout set to {0:d} ms", timeout)

        ' Set target power
        KomodoApi.km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, power)
        Console.WriteLine("Target power ON")

        ' Configure all GPIO pins as inputs
        For i = 0 To NUM_GPIOS
            KomodoApi.km_gpio_config_in(km, i, _
                                    km_pin_bias_t.KM_PIN_BIAS_PULLUP, _
                                    km_pin_trigger_t.KM_PIN_TRIGGER_BOTH_EDGES)
        Next
        Console.WriteLine("All pins set as inputs")

        candump(km, max_events)

        ' Close and exit
        KomodoApi.km_close(km)
    End Sub
End Module
