'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : request.vb
'--------------------------------------------------------------------------
' Requesting CAN data test routine
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES
' LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================
Imports TotalPhase

Module request

    '======================================================================
    ' UTILITY FUNCTIONS
    '======================================================================
    Sub print_num_array(ByRef array() As Byte, ByVal len As Integer)
        Dim i As Integer

        Console.Write("[ ")

        For i = 0 To len - 1
            Console.Write("{0:x} ", array(i))
        Next

        Console.Write("]")
        Console.Out.Flush()
    End Sub


    Sub _request(ByVal km As Integer, _
                 ByVal can_id As UInteger, _
                 ByVal ch As km_can_ch_t)
        Dim ret As Integer
        Dim arb_count As UInteger = 0
        Dim data_len As Byte = 8
        Dim data(data_len) As Byte

        Dim info As New KomodoApi.km_can_info_t
        Dim pkt As New KomodoApi.km_can_packet_t

        pkt.remote_req = 1
        pkt.extend_addr = 0
        pkt.dlc = data_len
        pkt.id = can_id

        ' Enable Komodo
        ret = KomodoApi.km_enable(km)
        If ret <> km_status_t.KM_OK Then
            Console.WriteLine("Unable to enable Komodo")
            Exit Sub
        End If

        Console.WriteLine("Request data from CAN ID {0:d} (0x{0:x})", can_id)

        ' Send a remote frame with no data.  5th arg is 0 and
        ' data array is not important.
        KomodoApi.km_can_write(km, ch, 0, pkt, 0, data, arb_count)

        ' Look for packet with desired CAN ID. Store in data[].
        pkt.id = can_id - 1 ' Set pkt_id <> can_id to start loop
        While pkt.id <> can_id
            ret = KomodoApi.km_can_read(km, info, pkt, data_len, data)
            If ((info.status And KomodoApi.KM_READ_TIMEOUT) <> 0) Then
                Console.WriteLine("Timeout: no data!")
                Console.WriteLine()
                Exit Sub
            End If
        End While

        Console.Write("Received {0:d} byte(s): ", ret)
        Console.Out.Flush()

        print_num_array(data, ret)
        Console.WriteLine()
        Console.WriteLine()

    End Sub

    '======================================================================
    ' MAIN PROGRAM
    '======================================================================
    Public Sub request_run(ByRef in_hex_str As String)
        Dim km As Integer
        Dim ret As Integer
        Dim power As km_power_t
        Dim port As Integer = 0
        Dim can_id As UInteger = Val(in_hex_str)
        Dim bitrate As UInteger = 125000 ' kHz
        Dim timeout As UInteger = 3000 ' ms

        port = 0
        power = km_power_t.KM_TARGET_POWER_ON

        ' Open the interface
        km = KomodoApi.km_open(port)
        If (km <= 0) Then
            Console.WriteLine("Unable to open Komodo on port {0:d}", port)
            Console.WriteLine("Error code = {0:d}", km)
            Exit Sub
        End If

        ' Acquire features to control and listen CAN A
        ret = KomodoApi.km_acquire(km, KomodoApi.KM_FEATURE_CAN_A_CONFIG Or _
                                       KomodoApi.KM_FEATURE_CAN_A_CONTROL Or _
                                       KomodoApi.KM_FEATURE_CAN_A_LISTEN)
        Console.WriteLine("Acquired features: 0x{0:x}", ret)

        ' Set bitrate
        ret = KomodoApi.km_can_bitrate(km, km_can_ch_t.KM_CAN_CH_A, bitrate)
        Console.WriteLine("Bitrate set to " & ret / 1000 & "kHz")

        ' Set timeout
        KomodoApi.km_timeout(km, timeout)
        Console.WriteLine("Timeout set to {0:d} ms", timeout)

        ' Set target power
        KomodoApi.km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, power)
        Console.WriteLine("Target power {0:s}", _
                      IIf(power = km_power_t.KM_TARGET_POWER_ON, "ON", "OFF"))
        Console.WriteLine()
        _request(km, can_id, km_can_ch_t.KM_CAN_CH_A)

        ' Close and exit
        KomodoApi.km_close(km)
    End Sub
End Module
