Attribute VB_Name = "async"
'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : async.bas
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================


'==========================================================================
' FUNCTIONS
'==========================================================================
Sub async(km As Long, can_id As Long, can_ch As Long, iter As Long)
    Dim i         As Long
    Dim ret       As Long
    Dim timeout   As Long
    Dim data(8)   As Byte
    Dim data_len  As Long
    Dim arb_count As Long

    For i = 1 To 8 ' Initalize array
       data(i) = CByte(i)
    Next

    timeout   = 500
    data_len  = 8
    arb_count = 0

    Dim pkt As km_can_packet_t
    pkt.remote_req  = 0
    pkt.extend_addr = 0
    pkt.dlc         = data_len
    pkt.id          = &HAB

    ' Enable Komodo
    ret = km_enable(km)
    If ret <> km_status_t.KM_OK Then
        Debug.Print "Unable to enable Komodo"
        Exit Sub
    End If

    ' Send first packet
    ret = km_can_async_submit(km, can_ch, 0, pkt, data_len, data)
    Debug.Print "Submitted packet 1"

    For i = 0 To iter - 2
        ' Send another packet while the first is still in progress
        ret = km_can_async_submit(km, can_ch, 0, pkt, data_len, data)
        Debug.Print "Submitted packet " & (i + 2)

        ' The application can now perform other functions
        ' while the Komodo is sending the previous packet
        km_sleep_ms (100)

        ' Collect response to previously submitted packet
        ret = km_can_async_collect(km, timeout, arb_count)

        If ret = km_status_t.KM_OK Then
            Debug.Print "Collected packet " & (i + 1)
        Else
            Debug.Print "Error " & ret
        End If
    Next

    ' Collect the last response
    ret = km_can_async_collect(km, timeout, arb_count)
    If ret = km_status_t.KM_OK Then
        Debug.Print "Collected packet " & (i + 1)
    Else
        Debug.Print "Error " & ret
    End If
End Sub


'==========================================================================
' MAIN PROGRAM
'==========================================================================
Public Sub async_run()
    Dim km      As Long
    Dim ret     As Long
    Dim port    As Long
    Dim iter    As Long
    Dim can_id  As Long
    Dim bitrate As Long

    port    = 0
    iter    = 10
    can_id  = 66
    bitrate = 125000 ' Hz

    ' Open the interface
    km = km_open(port)
    If km <= 0 Then
        Debug.Print "Unable to open Komodo on port " & port
        Debug.Print "Error code = " & km
        Exit Sub
    End If

    ' Aquire freatures
    ret = km_acquire(km, KM_FEATURE_CAN_A_CONFIG Or _
                         KM_FEATURE_CAN_A_CONTROL)
    Debug.Print "Aquired features: 0x" & Hex(ret)

    ' Set bitrate
    ret = km_can_bitrate(km, KM_CAN_CH_A, bitrate)
    Debug.Print "Bitrate set to " & ret / 1000 & "kHz, ";

    ' Set target power
    ret = km_can_target_power(km, KM_CAN_CH_A, KM_TARGET_POWER_ON)
    Debug.Print "Enabled target power for CAN A"

    Call async(km, can_id, km_can_ch_t.KM_CAN_CH_A, iter)
    Debug.Print

    ' Close and exit
    km_close (km)
End Sub
