Attribute VB_Name = "gpio"
'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : gpio.bas
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================

'==========================================================================
' FUNCTIONS
'==========================================================================
Sub show_pin_state(state As Long, pin As Long)
    Dim val As String
    Dim mask As Integer

    mask = (2 ^ (pin - 1))
    If state And mask Then
        val = "high"
    Else
        val = "low"
    End If

    Debug.Print "Pin " & pin & " is " & val
    km_sleep_ms (50)
End Sub

Public Sub monitor_changes(km As Long)
    Dim i         As Long
    Dim oldval    As Long
    Dim endtime   As Date
    Dim wait_sec  As Long
    Dim NUM_GPIOS As Long

    wait_sec = 20
    NUM_GPIOS = 8

    Debug.Print "Configuring pins as inputs with pullups"
    For i = 0 To NUM_GPIOS - 1
        ret = km_gpio_config_in(km, i, KM_PIN_BIAS_PULLUP, KM_PIN_TRIGGER_NONE)
    Next

    ' Enable Komodo
    km_enable (km)
    Debug.Print "Monitoring GPIO pin changes for " & wait_sec & " seconds:"

    endtime = DateAdd("s", wait_sec, Now)
    oldval = km_gpio_get(km)

    While Now() < endtime
        Dim change As Long
        Dim newval As Long

        ' Check for pin changes
        newval = km_gpio_get(km)

        If newval <> oldval Then
            change = oldval Xor newval

            ' Display the pins that changed
            For i = 0 To NUM_GPIOS - 1
                If change And (2 ^ i) Then
                    Call show_pin_state(newval, i + 1)
                End If
            Next

            ' Save the current state
            oldval = newval
        End If
    Wend
End Sub

Sub gpio(km As Long)
    Dim ret As Long

    Debug.Print "Configuring Pin 1 as an output"
    ret = km_gpio_config_out(km, KM_GPIO_PIN_1_CONFIG, KM_PIN_DRIVE_NORMAL, _
                             KM_PIN_SRC_SOFTWARE_CTL)

    Debug.Print "Configuring Pin 2 as an input with pullup"
    ret = km_gpio_config_in(km, KM_GPIO_PIN_2_CONFIG, KM_PIN_BIAS_PULLUP, _
                            KM_PIN_TRIGGER_NONE)

    ' Enable Komodo
    km_enable (km)

    Debug.Print "Setting Pin 1 to logic low"
    ret = km_gpio_set(km, 0, KM_GPIO_PIN_1_MASK)

    ' Show the state of each pin
    ' Pin 1 should be low because it was set low
    ' Pin 2 should be high because it's an input with a pullup resistor
    ret = km_gpio_get(km)
    Call show_pin_state(ret, 1)
    Call show_pin_state(ret, 2)

    ' Disable the Komodo to change the GPIO configuration
    km_disable (km)

    ' Configure pin 1 as an input with a pulldown resistor
    Debug.Print "Configuring Pin 2 as an input with pulldown"
    Call km_gpio_config_in(km, KM_GPIO_PIN_2_CONFIG, KM_PIN_BIAS_PULLDOWN, _
                           KM_PIN_TRIGGER_NONE)

    ' Enable and show value of Pin 2
    km_enable (km)
    ret = km_gpio_get(km)
    Call show_pin_state(ret, 2)
    km_disable (km)

    ' Set GPIOs as input and monitor for changes
    monitor_changes (km)
End Sub


'==========================================================================
' MAIN PROGRAM
'==========================================================================
Public Sub gpio_run()
    Dim km   As Long
    Dim ret  As Long
    Dim port As Long

    port = 0

    ' Open the interface
    km = km_open(port)
    If km <= 0 Then
        Debug.Print "Unable to open Komodo on port " & port
        Debug.Print "Error code = " & km
        Exit Sub
    End If

    ' Aquire freatures
    ret = km_acquire(km, KM_FEATURE_GPIO_LISTEN Or _
                         KM_FEATURE_GPIO_CONTROL Or _
                         KM_FEATURE_GPIO_CONFIG)
    Debug.Print "Aquired features: 0x" & Hex(ret)

    gpio (km)
    Debug.Print

    ' Close and exit
    km_close (km)
End Sub
