Attribute VB_Name = "loopback"
'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : loopback.bas
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================

'==========================================================================
' UTILITY FUNCTIONS
'==========================================================================
Sub print_num_array(data() As Byte, length As Long)
    Dim i As Byte

    Debug.Print "[ ";
    For i = 0 To length - 1
        Debug.Print Hex(data(i)) & " ";
    Next
    Debug.Print ("]")
End Sub

Sub loopback(kma As Long, kmb As Long)
    Dim i           As Long
    Dim ret         As Long
    Dim data_len    As Long
    Dim arb_count   As Long
    Dim data_in(8)  As Byte
    Dim data_out(8) As Byte

    data_len = 8
    arb_count = 0

    For i = 0 To 8 ' Initalize array
       data_out(i) = CByte(i + 1)
    Next

    Dim info As km_can_info_t

    Dim pkt As km_can_packet_t
    pkt.remote_req  = 0
    pkt.extend_addr = 0
    pkt.dlc         = data_len
    pkt.id          = &HAB

    ' Enable CAN A and B
    ret = km_enable(kma)
    If ret <> KM_OK Then
        Debug.Print "Unable to enable Komodo A"
        Exit Sub
    End If

    ret = km_enable(kmb)
    If ret <> KM_OK Then
        Debug.Print "Unable to enable Komodo B"
        Exit Sub
    End If

    ' Send data on CAN A
    ret = km_can_write(kma, KM_CAN_CH_A, 0, pkt, data_len, data_out, arb_count)

    Debug.Print
    Debug.Print "Sent data:     ";
    Call print_num_array(data_out, data_len)

    ' Read data on CAN B
    Do While ret = 0
        ret = km_can_read(kmb, info, pkt, data_len, data_in)
    Loop

    Debug.Print "Received data: ";
    Call print_num_array(data_in, ret)

    Debug.Print
    Debug.Print "Verifying data... "

    ' Verify received data
    For i = 0 To data_len - 1
        If data_out(i) <> data_in(i) Then
           Debug.Print "FAIL"
           Exit Sub
        End If
    Next

    Debug.Print "PASS"
End Sub


'==========================================================================
' MAIN PROGRAM
'==========================================================================
Public Sub loopback_run()
    Dim kma     As Long
    Dim kmb     As Long
    Dim reta    As Long
    Dim retb    As Long
    Dim porta   As Long
    Dim portb   As Long
    Dim bitrate As Long
    Dim timeout As Long

    porta = 0
    portb = 1
    timeout = 1000   ' ms
    bitrate = 125000 ' Hz

    kma = km_open(porta)
    If kma <= 0 Then
        Debug.Print "Unable to open Komodo on port " & porta
        Debug.Print "Error code = " & kma
        Exit Sub
    End If

    kmb = km_open(portb)
    If kmb <= 0 Then
        Debug.Print "Unable to open Komodo on port " & portb
        Debug.Print "Error code = " & kmb
        Exit Sub
    End If

    ' Setup CAN channel A for writing and B for listening
    reta = km_acquire(kma, KM_FEATURE_CAN_A_CONFIG Or _
                           KM_FEATURE_CAN_A_CONTROL)
    retb = km_acquire(kmb, KM_FEATURE_CAN_B_CONFIG Or _
                           KM_FEATURE_CAN_B_CONTROL Or _
                           KM_FEATURE_CAN_B_LISTEN)
    Debug.Print "Features for CAN A: 0x" & Hex(reta) & ", CAN B: 0x" & Hex(retb)

    ' Set bitrate
    reta = km_can_bitrate(kma, KM_CAN_CH_A, bitrate)
    retb = km_can_bitrate(kmb, KM_CAN_CH_B, bitrate)
    Debug.Print "Bitrate for CAN A set to " & reta / 1000 & "kHz, ";
    Debug.Print "CAN B set to " & retb / 1000 & "kHz"

    ' Set timeout
    ret = km_timeout(kmb, timeout)
    Debug.Print "Timeout for CAN B set to " & timeout & " ms"

    ' Set target power
    ret = km_can_target_power(kma, KM_CAN_CH_A, KM_TARGET_POWER_ON)
    ret = km_can_target_power(kmb, KM_CAN_CH_B, KM_TARGET_POWER_OFF)
    Debug.Print "Enabled target power for CAN A and disabled for CAN B"

    Call loopback(kma, kmb)
    Debug.Print

    ' Close and exit
    km_close (kma)
    km_close (kmb)
End Sub
