Attribute VB_Name = "monitor"
'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : monitor.bas
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================

Type CurrStruct
    value As Currency
End Type

'======================================================================
' UTILITY FUNCTIONS
'======================================================================
Function LongLongToText(ByRef value As LongLong) As String
    Dim L    As Long
    Dim val  As CurrStruct
    Dim Temp As String

    LSet val = value
    Temp = Format$(val.value, "#.0000")
    L = Len(Temp)
    Temp = Left$(Temp, L - 5) & Right$(Temp, 4)

    Do While Len(Temp) > 1 And Left$(Temp, 1) = "0"
        Temp = Mid$(Temp, 2)
    Loop

    Do While Len(Temp) > 2 And Left$(Temp, 2) = "-0"
        Temp = "-" & Mid$(Temp, 3)
    Loop

    LongLongToText = Temp
End Function

Function timestamp_to_ns(stamp As LongLong, samplerate_khz As Long) As LongLong
    Dim Temp   As CurrStruct
    Dim result As LongLong

    LSet Temp = stamp
    Temp.value = Temp.value * 1000000
    Temp.value = Temp.value / samplerate_khz
    LSet result = Temp
    timestamp_to_ns = result
End Function

Sub print_num_array(data() As Byte, length As Integer)
    Dim i As Byte

    For i = 0 To length - 1
        Debug.Print Hex(data(i)) & " ";
    Next
End Sub

Sub print_status(status As Long)
    Debug.Print " ";

    If (status = KM_OK) Then
        Debug.Print "OK ";
    End If
    If ((status And KM_READ_TIMEOUT) <> 0) Then
        Debug.Print "TIMEOUT ";
    End If
    If ((status And KM_READ_ERR_OVERFLOW) <> 0) Then
        Debug.Print "OVERFLOW ";
    End If
    If ((status And KM_READ_END_OF_CAPTURE) <> 0) Then
        Debug.Print "END OF CAPTURE ";
    End If
    If ((status And KM_READ_CAN_ARB_LOST) <> 0) Then
        Debug.Print "ARBITRATION LOST ";
    End If
    If ((status And KM_READ_CAN_ERR) <> 0) Then
        Debug.Print "ERROR " & Hex(status And KM_READ_CAN_ERR_FULL_MASK);
    End If
End Sub

Sub print_events(events As Long, bitrate As Long)
    If events = 0 Then
        Exit Sub
    End If
    If ((events And KM_EVENT_DIGITAL_INPUT) <> 0) Then
        Debug.Print "GPIO CHANGE 0x" & _
                    Hex(events And KM_EVENT_DIGITAL_INPUT_MASK);
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_LISTEN_ONLY) <> 0) Then
        Debug.Print "BUS STATE LISTEN ONLY; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_CONTROL) <> 0) Then
        Debug.Print "BUS STATE CONTROL; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_WARNING) <> 0) Then
        Debug.Print "BUS STATE WARNING; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_ACTIVE) <> 0) Then
        Debug.Print "BUS STATE ACTIVE; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_PASSIVE) <> 0) Then
        Debug.Print "BUS STATE PASSIVE; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_STATE_OFF) <> 0) Then
        Debug.Print "BUS STATE OFF; ";
    End If
    If ((events And KM_EVENT_CAN_BUS_BITRATE) <> 0) Then
        Debug.Print "BITRATE " & bitrate / 1000 & "kHz; ";
    End If
End Sub

Sub candump(km As Long, max_events As Long)
    Dim ret            As Long
    Dim count          As Long
    Dim data(8)        As Byte
    Dim tmp_str        As String
    Dim data_len       As Long
    Dim samplerate_khz As Long

    Dim pkt            As km_can_packet_t
    Dim info           As km_can_info_t

    data_len = 8

    ' Get samplerate
    samplerate_khz = km_get_samplerate(km) / 1000

    ' Enable Komodo
    ret = km_enable(km)
    If (ret <> KM_OK) Then
        Debug.Print "Unable to enable Komodo"
        Exit Sub
    End If

    ' Print description of csv output
    Debug.Print
    Debug.Print "index,time(ns),(status & events),<ID:rtr/data> hex data";
    Debug.Print

    ' Start monitoring.
    count = 0
    While ((max_events = 0) Or (count < max_events))
        count = count + 1
        ret = km_can_read(km, info, pkt, data_len, data)

        tmp_str = LongLongToText(timestamp_to_ns(info.timestamp, samplerate_khz))
        Debug.Print count & "," & tmp_str & ",(";

        If ret < 0 Then
            Debug.Print "error=" & ret
        End If

        Call print_status(info.status)
        Call print_events(info.events, info.bitrate_hz)
        Debug.Print ")";

        ' Continue printing if we didn't see timeout
        ' error or dataless (events)
        If ((info.status = KM_OK) And (info.events = 0)) Then
            tmp_str = IIf(pkt.remote_req, "rtr", "data")
            Debug.Print "<" & Hex(pkt.id) & ":" & tmp_str & ">";

            ' If packet contained data, print it
            If pkt.remote_req = 0 Then
                Call print_num_array(data, CInt(pkt.dlc))
            End If
        End If

        Debug.Print
    Wend
End Sub

'==========================================================================
' MAIN PROGRAM
'==========================================================================
Public Sub monitor_run()
    Dim i          As Long
    Dim km         As Long
    Dim ret        As Long
    Dim port       As Long
    Dim power      As Long
    Dim timeout    As Long
    Dim bitrate    As Long
    Dim NUM_GPIOS  As Integer
    Dim max_events As Long

    port = 0
    power = KM_TARGET_POWER_ON
    timeout = 1000      ' ms
    bitrate = 125000    ' Hz
    NUM_GPIOS = 8
    max_events = 100

    '  Open the interface
    km = km_open(port)
    If km <= 0 Then
        Debug.Print "Unable to open Komodo on port " & port
        Debug.Print "Error code = " & km
        Exit Sub
    End If

    ' Acquire features.  Acquiring KM_FEATURE_CAN_A_CONTROL causes the
    ' Komodo interface to ACK all packets transmitted on the bus.  Remove
    ' this feature to prevent the device from transmitting anything on the
    ' bus.
     ret = km_acquire(km, KM_FEATURE_CAN_A_CONFIG Or _
                          KM_FEATURE_CAN_A_LISTEN Or _
                          KM_FEATURE_CAN_A_CONTROL Or _
                          KM_FEATURE_GPIO_CONFIG Or _
                          KM_FEATURE_GPIO_LISTEN)
     Debug.Print "Acquired features 0x" & Hex(ret)

     ' Set bitrate
     ret = km_can_bitrate(km, KM_CAN_CH_A, bitrate)
     Debug.Print "Bitrate set to " & ret / 1000 & " kHz"

     ' Set timeout
     ret = km_timeout(km, timeout)
     Debug.Print "Timeout set to " & timeout & " ms"

     ' Set target power
     ret = km_can_target_power(km, KM_CAN_CH_A, power)
     Debug.Print "Target power ON"

     ' Configure all GPIO pins as inputs
     For i = 0 To NUM_GPIOS
         ret = km_gpio_config_in(km, i, KM_PIN_BIAS_PULLUP, KM_PIN_TRIGGER_BOTH_EDGES)
     Next

     Debug.Print "All pins set as inputs"

     Call candump(km, max_events)

     'Close and exit
     km_close (km)
End Sub
