Attribute VB_Name = "request"
'==========================================================================
' (c) 2011  Total Phase, Inc.
'--------------------------------------------------------------------------
' Project : Komodo Sample Code
' File    : request.bas
'--------------------------------------------------------------------------
' Redistribution and use of this file in source and binary forms, with
' or without modification, are permitted.
'
' THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
' "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
' FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
' COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
' INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
' BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
' LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
' CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
' LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
' ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
' POSSIBILITY OF SUCH DAMAGE.
'==========================================================================

'==========================================================================
' UTILITY FUNCTIONS
'==========================================================================
Sub print_num_array(data() As Byte, length As Long)
    Dim i As Byte

    Debug.Print "[ ";
    For i = 0 To length - 1
        Debug.Print Hex(data(i)) & " ";
    Next
    Debug.Print ("]")
End Sub

Sub request_data(km As Long, can_id As Long, ch As Long)
    Dim ret       As Long
    Dim data(8)   As Byte
    Dim data_len  As Long
    Dim arb_count As Long

    data_len  = 8
    arb_count = 0

    Dim info As km_can_info_t
    Dim pkt As km_can_packet_t
    pkt.remote_req  = 1
    pkt.extend_addr = 0
    pkt.dlc         = data_len
    pkt.id          = can_id

    ' Enable Komodo
    ret = km_enable(km)
    If ret <> km_status_t.KM_OK Then
        Debug.Print "Unable to enable Komodo"
        Exit Sub
    End If

    Debug.Print "Request data from CAN ID: " & can_id & _
                " (0x" & Hex(can_id) & ")"

    ' Send a remote frame with no data.  5th arg is 0 and
    ' data array is not important.
    ret = km_can_write(km, ch, 0, pkt, 0, data, arb_count)

    ' Look for packet with desired CAN ID. Store in data[].
    ' Set pkt_id <> can_id to start loop
    pkt.id = can_id - 1
    While pkt.id <> can_id
        ret = km_can_read(km, info, pkt, data_len, data)
        If ((info.status And KM_READ_TIMEOUT) <> 0) Then
            Debug.Print "Timeout: no data!"
            Debug.Print
            Exit Sub
        End If
    Wend

    Debug.Print "Received " & ret & " byte(s): "

    Call print_num_array(data, ret)
    Debug.Print
End Sub


'==========================================================================
' MAIN PROGRAM
'==========================================================================
Public Sub request_run(in_hex_str As String)
    Dim km      As Long
    Dim ret     As Long
    Dim port    As Long
    Dim power   As Long
    Dim can_id  As Long
    Dim bitrate As Long
    Dim timeout As Long
    Dim tmp_str As String

    port    = 0
    power   = KM_TARGET_POWER_ON
    can_id  = Val(in_hex_str)
    bitrate = 125000 ' kHz
    timeout = 3000 ' ms

    ' Open the interface
    km = km_open(port)
    If (km <= 0) Then
        Debug.Print "Unable to open Komodo on port " & port
            Debug.Print "Error code = " & km
            Exit Sub
    End If

    ' Acquire features to control and listen CAN A
    ret = km_acquire(km, KM_FEATURE_CAN_A_CONFIG Or _
                         KM_FEATURE_CAN_A_CONTROL Or _
                         KM_FEATURE_CAN_A_LISTEN)
    Debug.Print "Acquired features: 0x" & Hex(ret)

    ' Set bitrate
    ret = km_can_bitrate(km, KM_CAN_CH_A, bitrate)
    Debug.Print "Bitrate set to " & ret / 1000 & "kHz"

    ' Set timeout
    ret = km_timeout(km, timeout)
    Debug.Print "Timeout set to " & timeout & "ms"

    ' Set target power
    ret = km_can_target_power(km, km_can_ch_t.KM_CAN_CH_A, power)
    tmp_str = IIf(power = KM_TARGET_POWER_ON, "ON", "OFF")
    Debug.Print "Target power " & tmp_str

    Debug.Print
    Call request_data(km, can_id, km_can_ch_t.KM_CAN_CH_A)

    ' Close and exit
    km_close (km)
End Sub
